class InvalidDataSourceError(Exception):
    ''' Exception: The data source invalid '''

class InvalidSelectorError(Exception):
    ''' Exception: The selector invalid '''

class InvalidInputError(Exception):
    ''' Exception: Invalid input '''

class FileNotFoundError(Exception):
    ''' Exception: File not found '''

class LexicalError(Exception):
    ''' Exception: Selector Syntax Error '''